# Generated by sila2.code_generator; sila2.__version__: 0.12.2
# -----
# This class does not do anything useful at runtime. Its only purpose is to provide type annotations.
# Since sphinx does not support .pyi files (yet?), this is a .py file.
# -----

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:

    from typing import Iterable, Optional

    from sessionmanagement_types import SessionCreate_Responses, SessionDelete_Responses
    from sila2.client import ClientMetadataInstance


class SessionManagementClient:
    """

    The features allows you to take exclusive access to the Opentron, this
    is required to run any other commands.

    """

    def SessionCreate(self, *, metadata: Optional[Iterable[ClientMetadataInstance]] = None) -> SessionCreate_Responses:
        """
        Creates a session to allow for controlling the Opentrons

        """
        ...

    def SessionDelete(
        self, SessionToken: str, *, metadata: Optional[Iterable[ClientMetadataInstance]] = None
    ) -> SessionDelete_Responses:
        """
        This stops the session and disables access through SiLA2

        """
        ...



# Generated by sila2.code_generator; sila2.__version__: 0.12.2
from .labware_base import LabwareBase
from .labware_feature import LabwareFeature
from .labware_client import LabwareClient

from .labware_types import RegisterLabware_Responses

from .labware_types import DeleteLabware_Responses

from .labware_types import MoveLabware_Responses






__all__ = [
    "LabwareBase",
    "LabwareFeature",
    "LabwareClient",

    "RegisterLabware_Responses",

    "DeleteLabware_Responses",

    "MoveLabware_Responses",




]



# Generated by sila2.code_generator; sila2.__version__: 0.12.2
from __future__ import annotations


from typing import NamedTuple



    

    

    





class RegisterLabware_Responses(NamedTuple):
    
    
    ItemIdentifier: str
    """
    UUID identifier for referenceable labware, modules,
                adapters, etc.
    """
    

class DeleteLabware_Responses(NamedTuple):
    
    pass
    
    

class MoveLabware_Responses(NamedTuple):
    
    pass
    
    






import uuid
from threading import Lock
from typing import Dict

from opentrons.protocol_api import ProtocolContext, InstrumentContext, ModuleContext, HeaterShakerContext, TrashBin

from .labware.labware_wrapper import LabwareWrapper


class Base:
    def __init__(self):
        self._lock = Lock()
        self.session_id: str = ""
        self.protocol: ProtocolContext | None = None
        self.labwares: Dict[str, LabwareWrapper] = {}
        self.pipette: InstrumentContext | None = None
        self.heaterShaker: HeaterShakerContext | None = None
        self.trash: TrashBin | None = None

    """Session"""

    def start_session(self) -> str:
        self._lock.acquire(blocking=True)
        if self.session_id == "":
            self.session_id = str(uuid.uuid4())
            self._lock.release()
            return self.session_id
        else:
            self._lock.release()
            return ""

    def end_session(self, session_id: str) -> None:
        self._lock.acquire(blocking=True)
        if self.session_id == session_id:
            self.protocol.cleanup()
            self.labwares.clear()
            self.pipette = None
            self.heaterShaker = None
            self.trash = None
            self.protocol = None
            self.session_id = ""
        self._lock.release()

    def get_session_id(self) -> str | None:
        self._lock.acquire(blocking=True)
        if self.session_id == "":
            self._lock.release()
            return None
        new_session_id = str(self.session_id)
        self._lock.release()
        return new_session_id

    def set_session_protocol(self, protocol: ProtocolContext) -> None:
        self._lock.acquire(blocking=True)
        self.protocol = protocol
        self._lock.release()

    """Labware"""

    def add_labware(self, labware: LabwareWrapper, identifier: str):
        self._lock.acquire(blocking=True)
        self.labwares[identifier] = labware
        self._lock.release()

    def get_labware(self, identifier: str) -> LabwareWrapper:
        return self.labwares[identifier]

    def get_labware_by_position(self, position: str) -> LabwareWrapper | None:
        for labware in self.labwares.values():
            if labware.position == position:
                return labware
        return None

    def delete_labware(self, identifier: str):
        self._lock.acquire(blocking=True)
        self.labwares.pop(identifier)
        self._lock.release()

    """Temporary Setup Methods"""

    def set_instrument(self, instrument: InstrumentContext):
        self._lock.acquire(blocking=True)
        self.pipette = instrument
        self._lock.release()

    def get_instrument(self) -> InstrumentContext:
        return self.pipette

    def set_module(self, module: ModuleContext):
        self._lock.acquire(blocking=True)
        if module is HeaterShakerContext:
            self.heaterShaker = module
        self._lock.release()

    def get_module(self) -> ModuleContext:
        return self.heaterShaker

    def set_trash(self, trash: TrashBin):
        self._lock.acquire(blocking=True)
        self.trash = trash
        self._lock.release()

    def get_trash(self) -> TrashBin:
        return self.trash



# Generated by sila2.code_generator; sila2.__version__: 0.12.2
from __future__ import annotations

from abc import ABC, abstractmethod
from typing import TYPE_CHECKING

from sila2.server import FeatureImplementationBase, MetadataDict

from .sessionmanagement_types import SessionCreate_Responses, SessionDelete_Responses

if TYPE_CHECKING:

    from ...server import Server


class SessionManagementBase(FeatureImplementationBase, ABC):
    parent_server: Server

    def __init__(self, parent_server: Server):
        """

        The features allows you to take exclusive access to the Opentron, this
        is required to run any other commands.

        """
        super().__init__(parent_server=parent_server)

    @abstractmethod
    def SessionCreate(self, *, metadata: MetadataDict) -> SessionCreate_Responses:
        """
        Creates a session to allow for controlling the Opentrons



        :param metadata: The SiLA Client Metadata attached to the call

        :return:

            - SessionToken: The session token used for running other commands



        """

    @abstractmethod
    def SessionDelete(self, SessionToken: str, *, metadata: MetadataDict) -> SessionDelete_Responses:
        """
        This stops the session and disables access through SiLA2



        :param SessionToken: The session token used for running other commands


        :param metadata: The SiLA Client Metadata attached to the call

        """
